; source for the controller-board (www.frydaysoft.de)
;----------------------------------------------------
;
; LCD-display functions
;----------------------------------------------------------------------
;put const-string to LCD
lcdstring:				
	CLR A
	MOVC A,@A+DPTR				;get char
	CJNE A,#ENDE,lcdstring2		;check for end-byte
	JMP lcdstringret
lcdstring2:
	call write_char
	JNB step_walk,lcdstring3
	CALL wait
lcdstring3:
	CALL shortwait
	CALL LCDBusy
	JC lcdstring3				;wait until LCD is ready
	INC DPTR
	JMP lcdstring
lcdstringret:
	CLR step_walk
	RET

;----------------------------------------------------------------------
;put string(lcd_buffer) to LCD
WriteLCD_Array:
	mov a,tmppointer
	PUSH acc
	MOV tmppointer,#lcd_buffer
WriteLCD1:
	MOV A,@tmppointer
	CJNE A,#ENDE,WriteLCD2			;check for end
	JMP WriteLCDRet
WriteLCD2:
	MOV A,@tmppointer
	call write_char	
	INC tmppointer
	JNB step_walk,WriteLCD3
	CALL wait
WriteLCD3:
	CALL shortwait
	CALL LCDBusy
	JC WriteLCD3					;check if busy
	MOV A,tmppointer
	CLR C
	SUBB A,#lcd_buffer              
	CJNE A,#LCD_BUFMAX,WriteLCD1	;Array_overrun?
WriteLCDRet:
	CLR step_walk
	POP acc
	mov tmppointer,a
	RET

;--------------------------------------------
;put byte to LCD
;acc -> data 
write_char:
	PUSH DPL
	PUSH DPH
	MOV DPL,#LCD_DataWR
	ANL DPL,#0fh
	MOV DPH,#LCD_DataWR
	ANL DPH,#0f0h
	MOVX @DPTR,A
	CALL getCursor
	CJNE A,#ADR_12LCD,wr_ch_end2
	MOV A,#ADR_2LCD					;cursor to 2. line/bank
	CALL setcursor
	JMP wr_char_ret
wr_ch_end2:
	CJNE A,#ADR_22LCD,wr_char_ret
	MOV A,#ADR_1LCD					;cursor to home
	CALL setcursor
	JMP wr_char_ret
wr_char_ret:
	POP DPH
	POP DPL
	RET

;--------------------------------------------
;reads LCD-Contr. Carry=1 if busy
LCDBusy:        
	;anlernative code  -> only short waitcycle
	;call shortwait	;wait a little time
	;clr c		;LCD should be ready
	;ret
	PUSH DPL
	PUSH DPH
	MOV DPL,#LCD_RegRD
	ANL DPL,#0fh
	MOV DPH,#LCD_RegRD
	ANL DPH,#0f0h
	MOVX A,@DPTR
	CLR C
	ANL A,#80h
	JZ LCDBusyRet
	SETB C
LCDBusyRet:
	POP DPH
	POP DPL
	RET
	
;--------------------------------------------
; set cursor in LCD-module
; in acc
setcursor:
	PUSH DPL
	PUSH DPH
	MOV DPL,#LCD_RegWR
	ANL DPL,#0fh
	MOV DPH,#LCD_RegWR
	ANL DPH,#0f0h
	ORL A,#80h				;set command-bit
	MOVX @DPTR,A			;set cursor
	POP DPH
	POP DPL
	RET

;--------------------------------------------
;read RAM-addr of LCD and return it in acc (0-127)
;out acc
getCursor:
	PUSH DPL
	PUSH DPH
	MOV DPL,#LCD_RegRD
	ANL DPL,#0fh
	MOV DPH,#LCD_RegRD
	ANL DPH,#0f0h
	MOVX A,@DPTR
	ANL A,#7Fh				;clear busy-flag
	POP DPH
	POP DPL
	RET

;--------------------------------------------
; set cursor home and clear LCD-module
clearLCD:
	PUSH DPL
	PUSH DPH
	MOV A,#1h               ;homecursor and clear

	MOV DPL,#LCD_RegWR
	ANL DPL,#0fh
	MOV DPH,#LCD_RegWR
	ANL DPH,#0f0h
	MOVX @DPTR,A
clear_busy:
	CALL shortwait
	CALL LCDBusy
	JC clear_busy
	POP DPH
	POP DPL
	RET

;-----------------------------------------------------------------
;init LCD-modul
lcd_init:
	MOV tmppointer,#2
init_again:
	CALL shortwait				;001 DL N F * *  DL:8/4 Bit N:0=one/1=two-lines F:0=5x7/1=5x10Dots
	MOV A,#00111000b			;8bit-mode one line 5x7dots
	MOV B,#LCD_RegWR
	LCALL out_io
	LCALL out_io
	CALL shortwait
	DJNZ tmppointer,init_again	;call 2x init-sequence

	JB blick_cursor,b_cursor	
	MOV A,#00001100b      		;LCD on no Cursor
	JMP after_cur
b_cursor:
	MOV A,#00001101b      		;LCD on no Cursor
after_cur:
	MOV B,#LCD_RegWR
	LCALL out_io
	LCALL shortwait
	MOV A,#00000001b			;clear lcd
	MOV B,#LCD_RegWR
	LCALL out_io
	LCALL shortwait
	MOV A,#00000110b			;set corsor-direction auto-dec
	MOV B,#LCD_RegWR
	LCALL out_io
	LCALL shortwait
	CLR blick_cursor
	RET

;--------------------------------------------

