; example for the controller-board (www.frydaysoft.de)
;flash one Leds and put pressed KEY to LED
;poll the HCT245 and put this data to HCT574
;put text to LCD-display

$include (mod51)

;**** Bit-RAM 20..2F ****
 BSEG AT 02Fh		; use only last byte of BSEG-block     
  blick_cursor: DBIT 1	; param for init_lcd
  step_walk:    DBIT 1  ; param for writeLCD_Array

;**** Byte-RAM from 8h upto 2Dh ****
 DSEG AT 8h		; bank1..3 8..1Fh
  lcd_buffer:  DS 10h   ; string-buffer 16 byte for LCD-Output
  unused_buf:  DS  8h	; 8 byte unused and BSEG not complete used
 DSEG AT 30h		; 30..7Fh
  STACK_RAM:   DS 50h	; **** reserved STACK-area  ****


;**** Reg-zuweisung ****
Key4	equ P1.0
Key3	equ P1.1
Key2	equ P1.2
Key1	equ P1.3
Led4	equ P1.4
Led3	equ P1.5
Led2	equ P1.6
Led1	equ P1.7

tmppointer	equ R0
;R1..R7 not used

waitconst 	equ   0h       	;const for wait-looping
Stack_First 	equ  30h     	;set stackspointer to reserved RAM
baud 		equ  250        ;timer1 -> baud=256-fosz/(Baud-rate*12*16)
ADR_2LCD	equ   40	;define adr. of the 2. LCD-bank
ENDE 		equ 0ffh	;define for end of a string

;***Pins of HCT138 (IC6)***********************************************************************

LCD_RegWR 	equ  80h       	;write lcd-register A0=0 ,A1=0
LCD_RegRD 	equ  81h       	;read  lcd-register A0=1 ,A1=0
LCD_DataWR 	equ  82h       	;write lcd-data	    A0=0 ,A1=0
LCD_DataRD 	equ  83h       	;read  lcd-data	    A0=1 ,A1=0
HCT138_Y1	equ  10010000b
HCT138_Y2	equ  10100000b
HCT138_Y3	equ  10110000b
HCT138_Y4	equ  11000000b
HCT138_Y5	equ  11010000b
HCT138_Y6	equ  11100000b
HCT138_Y7	equ  11110000b

;##################### MAIN Program #######################################

cseg at 0
	;USING 3                 
	;USING 2                 
	;USING 1                 
	USING 0     	;Bank 0

org 0000h
jmp main_start		;jump to program

org 0003h               ;extInt0
reti

org 000bh               ;timer0
reti

org 0013h               ;extInt1
reti

org 001Bh               ;timer1
reti

org 0023h               ;RS232
reti
;jmp SIO_INT

;**************************************************************************

main_start:
	mov SP,  #Stack_First
	mov DPTR,#0
	mov acc, #0
	mov b,	 #0
	mov PSW, #0
	mov PCON,#0
	mov SCON,#0
	mov TCON,#0
	mov TMOD,#0
	mov IE,  #0
	mov IP,  #0
	clr step_walk
	setb blick_cursor
	call lcd_init
		;lcall init_com         	;init V.24 at 9600 Baud
	mov DPTR,#LCD_TEXT		;alernative code
	setb step_walk
	call lcdstring			;with defined const-string

	mov a,#44			;reposition cursor
	call setcursor

	mov lcd_buffer+ 0,#50		;define dynamic LCD-string
	mov lcd_buffer+ 1,#48
	mov lcd_buffer+ 2,#48
	mov lcd_buffer+ 3,#48
	mov lcd_buffer+ 4,#ENDE		;sign end of string
	setb step_walk
	call WriteLCD_Array		;put string to LCD


	mov P1,#0efh		;Led4 on
		;mov a,#0	;clear LCD
		;call setcursor
		;mov DPTR,#LCD_EMPTY
		;call WriteLCD

;################## Main - L O O P ########################

MainLoop:
	mov b,#HCT138_Y2	;read from HCT245
	call in_io
	cpl a
	mov b,#HCT138_Y1	;write to HCT574
	call out_io
	clr LED1		;set LED1,2,3 in case of KEY1,2,3
	jnb KEY1,ReadKey2
	setb LED1
ReadKey2:
	clr LED2
	jnb KEY2,ReadKey3	
	setb LED2
ReadKey3:
	clr LED3
	jnb KEY3,ReadKey4
	setb LED3
ReadKey4:			
	jnb KEY4,call_wait	;stop toggle-mode if KEY4 is pressed
	jnb LED4,SetLed4	;toggle LED4		
	clr LED4
	jmp call_wait
SetLed4:
	setb LED4	
call_wait:
	call wait		;wait a little time
        	;jb RI,GetRi	;check recieved byte
        jmp MainLoop           	;nothings to do
GetRi:
        clr TI
        jnb TI,$		;return received byte

	;mov lcd_buffer,a		;put received byte
	;mov lcd_buffer+1,#ENDE		;to LCD and set 
	;call WriteLCD			;cursor to first position
	;mov a,#0
	;call setcursor

        jmp MainLoop

;----------------------------------------------------------------------
lcdstring:				;put const-string to LCD
	mov tmppointer,#0		;use tmppointer for counting
lcdstring0:
	cjne tmppointer,#11h,lcdstring1
	jmp lcdstringret
lcdstring1:
	mov a,tmppointer
	movc a,@a+DPTR			;calculate text-index
	cjne a,#ENDE,lcdstring2		;check for end-byte
	jmp lcdstringret
lcdstring2:
	mov b,#LCD_DataWR		;put byte to LCD
	call out_io
	jnb step_walk,lcdstring3
	call wait
lcdstring3:
	call shortwait
	mov b,#LCD_RegRD
	call LCDBusy
	jc lcdstring3			;wait until LCD is ready
	inc tmppointer
	cjne tmppointer,#8h,lcdstring0	;jump in next LCD-RAM-bank
	mov a,#ADR_2LCD			;if 8 byte reached
	call setcursor
lcdstring4:
	call shortwait
	mov b,#LCD_RegRD
	call LCDBusy
	jc lcdstring4			;wait until LCD-cursor is set
	jmp lcdstring0
lcdstringret:
	clr step_walk
	ret

;----------------------------------------------------------------------

WriteLCD_Array:
	mov tmppointer,#lcd_buffer
WriteLCD0:
	mov a,@tmppointer
	cjne a,#ENDE,WriteLCD1	;check for end
	jmp WriteLCDRet
WriteLCD1:
	mov a,@tmppointer
	mov b,#LCD_DataWR
	call out_io
	inc tmppointer
	jnb step_walk,WriteLCD2
	call wait
WriteLCD2:
	call shortwait
	mov b,#LCD_RegRD
	call LCDBusy
	jc WriteLCD2	;check if busy
	mov a,tmppointer
	clr c
	subb a,#lcd_buffer              ;Array_overrun?
	cjne a,#8,WriteLCDoverrun
	mov a,#ADR_2LCD
	call setcursor
WriteLCD3:
	call shortwait
	mov b,#LCD_RegRD
	lcall LCDBusy
	jc WriteLCD3
	jmp WriteLCD0
WriteLCDoverrun:
	cjne a,#11h,WriteLCD0 
WriteLCDRet:
	clr step_walk
	ret

;-----------------------------------------------------------------

lcd_init:
	mov tmppointer,#2
init_again:
	call shortwait
	mov a,#00111000b		;8bit-mode one line 5x7dots
	mov b,#LCD_RegWR
	lcall out_io
	lcall out_io
	call shortwait
	djnz tmppointer,init_again	;call 2x init-sequence

	jb blick_cursor,b_cursor	
	mov a,#00001100b      		;LCD on no Cursor
	jmp after_cur
b_cursor:
	mov a,#00001101b      		;LCD on no Cursor
after_cur:
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	mov a,#00000001b		;clear lcd
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	mov a,#00000110b		;set corsor-direction auto-dec
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	clr blick_cursor
	ret

;--------------------------------------------

LCDBusy:        ;reads LCD-Contr. Carry=1 if busy

	;xxl    anlernative code  -> only short waitcycle
	;call shortwait	;wait a little time
	;clr c		;LCD should be ready
	;ret

	push dpl
	push dph
	mov dpl,b
	anl dpl,#0fh
	mov dph,b
	anl dph,#0f0h
	movx a,@DPTR
	clr c
	anl a,#80h
	jz LCDBusyRet
	setb c
LCDBusyRet:
	pop dph
	pop dpl
	ret
	
;--------------------------------------------

setcursor:
	push dpl
	push dph
	push acc		;store cursor-value
	mov a,#2h               ;homecursor 1.64ms
	mov dpl,#LCD_RegWR
	anl dpl,#0fh
	mov dph,#LCD_RegWR
	anl dph,#0f0h
	movx @DPTR,a
cursor_busy:
	call shortwait
	mov b,#LCD_RegRD
	call LCDBusy
	jc cursor_busy
	pop b			;get cursor-value
setcursor0:
	mov a,b
	jz setcursorRet		;check acc if zero
	mov a,#00010100b        ;step cursor right
	mov dpl,#LCD_RegWR
	anl dpl,#0fh
	mov dph,#LCD_RegWR
	anl dph,#0f0h
	movx @DPTR,a
	call shortwait
	djnz b,setcursor0
setcursorRet:
	pop dph
	pop dpl
        ret

;--------------------------------------------

shortwait:
	push acc
	mov a,#0
shortwait0:
	nop
	nop
	nop
	djnz acc,shortwait0
	pop acc
	ret

;--------------------------------------------
 
wait:
	mov a,#waitconst
wait0:
	call shortwait
	djnz acc,wait0
	ret

;--------------------------------------------
;acc -> data 
;b   -> IO-high-adresse
out_io:
	push dpl
	push dph
	mov dpl,b
	anl dpl,#0fh
	mov dph,b
	anl dph,#0f0h
	movx @DPTR,a
	pop dph
	pop dpl
	ret

;--------------------------------------------
;return data in acc 
;b   -> IO-high-adresse
in_io:
	push dpl
	push dph
	mov dpl,b
	anl dpl,#0fh
	mov dph,b
	anl dph,#0f0h
	movx a,@DPTR
	pop dph
	pop dpl
	ret

;--------------------------------------------

init_com:
	orl PCON,#80h      ; normal or faster processing
	orl SCON,#50h      ; SCON-Mode 1 (8Bit-UART) and REN 
	orl TCON,#40h      ; setze TR1 ->use TIMER1 (xxx1xxxx)
	orl TMOD,#20h      ; set 8Bit-auto-reload-TIMER0  (00100000)
	mov TH1,#baud      ; Timer1 fr Baud-rate inititialisieren
	mov TL1,#baud
	setb SM2
	clr TF1
	setb TR1
        clr ET1
        	;setb ES
	clr RI             ;erlaube empfang
	clr PS		   ;lower priority
	ret                


;################ Interrupts ###################

SIO_INT:
        jnb TI,SIO_Int0
        setb TI
        clr TI
SIO_Int0:
        jnb RI,Int_SIOExit
       clr RI
       push acc
       mov a,SBUF
       pop acc
Int_SIOExit:
        reti

;****************************************************************

LCD_EMPTY:     DB '                ',ENDE
LCD_TEXT:      DB 'FrydaySoft c1999',ENDE

end
